@@ -1,4 +1,5 @@
 import numpy as np
+import copy
 
 # astra_cuda_recon_algorithm_kwargs, tomopy_recon_algorithm_kwargs,
 # and tomopy_filter_names, extend_description_style
@@ -7,6 +8,7 @@ from ipywidgets import *
 from tomopy.recon.rotation import find_center_vo, find_center, find_center_pc
 from tomopyui.widgets.view import BqImViewer_Center, BqImViewer_Center_Recon
 from tomopyui.backend.util.center import write_center
+from tomopyui.widgets.helpers import ReactiveTextButton
 
 
 class Center:
@@ -42,24 +44,24 @@ class Center:
     """
 
     def __init__(self, Import):
-
         self.Import = Import
+        self.projections = Import.projections
         self.Import.Center = self
         self.current_center = self.Import.prenorm_uploader.projections.pxX / 2
         self.center_guess = None
         self.index_to_try = None
-        self.search_step = 0.5
-        self.search_range = 5
+        self.search_step = 10
+        self.search_range = 100
         self.cen_range = None
+        self.use_ds = True
         self.num_iter = int(1)
         self.algorithm = "gridrec"
         self.filter = "parzen"
         self.metadata = {}
-        self.projection_viewer = BqImViewer_Center(self)
-        self.projection_viewer.create_app()
+        self.viewer = BqImViewer_Center()
+        self.viewer.create_app()
         self.rec_viewer = BqImViewer_Center_Recon()
         self.rec_viewer.create_app()
-
         self._init_widgets()
         self._set_observes()
         self.make_tab()
@@ -68,7 +70,6 @@ class Center:
         """
         Sets `Center` metadata.
         """
-
         self.metadata["center"] = self.current_center
         self.metadata["center_guess"] = self.center_guess
         self.metadata["index_to_try"] = self.index_to_try
@@ -99,13 +100,26 @@ class Center:
             disabled=False,
             style=extend_description_style,
         )
-        self.find_center_button = Button(
-            description="Click to automatically find center (image entropy).",
-            disabled=False,
-            button_style="info",
-            tooltip="",
-            icon="",
-            layout=Layout(width="auto", justify_content="center"),
+        self.find_center_button = ReactiveTextButton(
+            self.find_center_on_click,
+            "Click to automatically find center (image entropy).",
+            "Automatically finding center (image entropy).",
+            "Found center.",
+            warning="Please import some data first.",
+        )
+        self.find_center_vo_button = ReactiveTextButton(
+            self.find_center_vo_on_click,
+            "Click to automatically find center (Vo).",
+            f"Automatically finding center using slice {self.index_to_try}.",
+            "Found center.",
+            warning="Please import some data first.",
+        )
+        self.find_center_manual_button = ReactiveTextButton(
+            self.find_center_manual_on_click,
+            "Click to find center by plotting.",
+            f"Reconstructing slice {self.index_to_try}.",
+            "Now search for the center using the slider above.",
+            warning="Your projections do not have associated theta values.",
         )
         self.index_to_try_textbox = IntText(
             description="Slice to use: ",
@@ -141,42 +155,35 @@ class Center:
             value=self.filter,
             description="Filter:",
         )
-        self.find_center_vo_button = Button(
-            description="Click to automatically find center (Vo).",
-            disabled=False,
-            button_style="info",
-            tooltip="Vo's method",
-            icon="",
-            layout=Layout(width="auto", justify_content="center"),
-        )
-        self.find_center_manual_button = Button(
-            description="Click to find center by plotting.",
-            disabled=False,
-            button_style="info",
-            tooltip="Start center-finding reconstruction with this button.",
-            icon="",
-            layout=Layout(width="auto", justify_content="center"),
+        self.use_ds_checkbox = Checkbox(
+            description="Use viewer downsampling: ",
+            value=True,
+            disabled=True,
+            style=extend_description_style,
         )
 
+    def _use_ds_data(self, change):
+        self.use_ds = self.use_ds_checkbox.value
+
     def _center_update(self, change):
         self.current_center = change.new
         self.center_guess = change.new
-        self.projection_viewer.center_line.x = [
-            change.new / self.projection_viewer.pxX,
-            change.new / self.projection_viewer.pxX,
+        self.viewer.center_line.x = [
+            change.new / self.viewer.pxX,
+            change.new / self.viewer.pxX,
         ]
         self.set_metadata()
 
     def _center_guess_update(self, change):
         self.center_guess = change.new
-        self.projection_viewer.center_line.x = [
-            change.new / self.projection_viewer.pxX,
-            change.new / self.projection_viewer.pxX,
+        self.viewer.center_line.x = [
+            change.new / self.viewer.pxX,
+            change.new / self.viewer.pxX,
         ]
         self.set_metadata()
 
     def _load_rough_center_onclick(self, change):
-        self.center_guess = self.Import.projections.pxX / 2
+        self.center_guess = self.projections.pxX / 2
         self.current_center = self.center_guess
         self.center_textbox.value = self.center_guess
         self.center_guess_textbox.value = self.center_guess
@@ -210,125 +217,23 @@ class Center:
         self.filter = change.new
         self.set_metadata()
 
+    def _slice_slider_update(self, change):
+        self.viewer.slice_line.y = [
+            change.new / self.viewer.pxY,
+            change.new / self.viewer.pxY,
+        ]
+        self.index_to_try_textbox.value = int(change.new)
+
     def _center_textbox_slider_update(self, change):
         self.center_textbox.value = self.cen_range[change.new]
         self.center_guess_textbox.value = self.cen_range[change.new]
-        self.projection_viewer.center_line.x = [
-            self.cen_range[change.new] / self.projection_viewer.pxX,
-            self.cen_range[change.new] / self.projection_viewer.pxX,
+        self.viewer.center_line.x = [
+            self.cen_range[change.new] / self.viewer.pxX,
+            self.cen_range[change.new] / self.viewer.pxX,
         ]
         self.current_center = self.center_textbox.value
-
         self.set_metadata()
 
-    def find_center_on_click(self, change):
-        """
-        Callback to button for attempting to find center automatically using
-        `tomopy.recon.rotation.find_center`. Takes index_to_try and center_guess.
-        This method has worked better for me, if I use a good index_to_try
-        and center_guess.
-        """
-        prj_imgs = self.Import.projections.prj_imgs
-        angles_rad = self.Import.projections.angles_rad
-
-        self.find_center_button.button_style = "info"
-        self.find_center_button.icon = "fa-spin fa-cog fa-lg"
-        self.find_center_button.description = "Importing data..."
-        if self.Import.projections.imported is True:
-            prj_imgs = self.Import.projections.prj_imgs
-            angles_rad = self.Import.projections.angles_rad
-            self.Import.log.info("Finding center...")
-            self.Import.log.info(f"Using index: {self.index_to_try}")
-            self.find_center_button.description = "Finding center..."
-            self.find_center_button.button_style = "info"
-            self.current_center = find_center(
-                prj_imgs,
-                angles_rad,
-                ratio=0.9,
-                ind=self.index_to_try,
-                init=self.center_guess,
-            )
-            self.center_textbox.value = self.current_center
-            self.Import.log.info(f"Found center. {self.current_center}")
-            self.find_center_button.description = "Found center."
-            self.find_center_button.icon = "fa-check-square"
-            self.find_center_button.button_style = "success"
-        else:
-            self.find_center_button.description = "Please import some data first."
-            self.find_center_button.icon = "exclamation-triangle"
-            self.find_center_button.button_style = "warning"
-
-    def find_center_vo_on_click(self, change):
-        """
-        Callback to button for attempting to find center automatically using
-        `tomopy.recon.rotation.find_center_vo`. Note: this method has not worked
-        well for me.
-        """
-        prj_imgs = self.Import.projections.prj_imgs
-        angles_rad = self.Import.projections.angles_rad
-        self.find_center_vo_button.button_style = "info"
-        self.find_center_vo_button.icon = "fa-spin fa-cog fa-lg"
-        self.find_center_vo_button.description = "Importing data..."
-        if self.Import.projections.imported is True:
-            self.Import.log.info("Finding center using Vo method...")
-            self.Import.log.info(f"Using index: {self.index_to_try}")
-            self.find_center_vo_button.description = "Finding center using Vo method..."
-            self.find_center_vo_button.button_style = "info"
-            self.current_center = find_center_vo(prj_imgs, ncore=1)
-            self.center_textbox.value = self.current_center
-            self.Import.log.info(f"Found center. {self.current_center}")
-            self.find_center_vo_button.description = "Found center."
-            self.find_center_vo_button.icon = "fa-check-square"
-            self.find_center_vo_button.button_style = "success"
-        else:
-            self.find_center_vo_button.description = "Please import some data first."
-            self.find_center_vo_button.button_style = "warning"
-            self.find_center_vo_button.icon = "exclamation-triangle"
-
-    def find_center_manual_on_click(self, change):
-        """
-        Reconstructs at various centers when you click the button, and plots
-        the results with a slider so one can view. TODO: see X example.
-        Uses search_range, search_step, center_guess.
-        Creates a :doc:`hyperslicer <mpl-interactions:examples/hyperslicer>` +
-        :doc:`histogram <mpl-interactions:examples/hist>` plot
-        """
-        self.find_center_manual_button.button_style = "info"
-        self.find_center_manual_button.icon = "fas fa-cog fa-spin fa-lg"
-        self.find_center_manual_button.description = "Starting reconstruction."
-
-        prj_imgs = self.Import.projections.prj_imgs
-        angles_rad = self.Import.projections.angles_rad
-        cen_range = [
-            self.center_guess - self.search_range,
-            self.center_guess + self.search_range,
-            self.search_step,
-        ]
-
-        # reconstruct, but also pull the centers used out to map to center
-        # textbox
-        self.rec, self.cen_range = write_center(
-            prj_imgs,
-            angles_rad,
-            cen_range=cen_range,
-            ind=self.index_to_try,
-            mask=True,
-            algorithm=self.algorithm,
-            filter_name=self.filter,
-            num_iter=self.num_iter,
-        )
-        if self.rec is None:
-            self.find_center_manual_button.button_style = "warning"
-            self.find_center_manual_button.icon = ""
-            self.find_center_manual_button.description = (
-                "Your projections do not have associated theta values."
-            )
-
-        self.rec_viewer.plot(self.rec)
-        self.find_center_manual_button.button_style = "success"
-        self.find_center_manual_button.icon = "fa-check-square"
-        self.find_center_manual_button.description = "Finished reconstruction."
-
     def _set_observes(self):
         self.center_textbox.observe(self._center_update, names="value")
         self.center_guess_textbox.observe(self._center_guess_update, names="value")
@@ -339,16 +244,104 @@ class Center:
         self.search_step_textbox.observe(self._search_step_update, names="value")
         self.algorithms_dropdown.observe(self._update_algorithm, names="value")
         self.filters_dropdown.observe(self._update_filters, names="value")
-        self.find_center_button.on_click(self.find_center_on_click)
-        self.find_center_vo_button.on_click(self.find_center_vo_on_click)
-        self.find_center_manual_button.on_click(self.find_center_manual_on_click)
         # Callback for index going to center
         self.rec_viewer.image_index_slider.observe(
             self._center_textbox_slider_update, names="value"
         )
+        self.viewer.slice_line_slider.observe(self._slice_slider_update, names="value")
+        self.use_ds_checkbox.observe(self._use_ds_data, names="value")
+
+    def find_center_on_click(self, *args):
+        """
+        Callback to button for attempting to find center automatically using
+        `tomopy.recon.rotation.find_center`. Takes index_to_try and center_guess.
+        This method has worked better for me, if I use a good index_to_try
+        and center_guess.
+        """
+        prj_imgs, ds_value = self.get_ds_projections()
+        angles_rad = self.projections.angles_rad
+        self.current_center = find_center(
+            prj_imgs,
+            angles_rad,
+            ratio=0.9,
+            ind=self.index_to_try,
+            init=self.center_guess,
+        )
+        self.center_textbox.value = self.current_center
+
+    def find_center_vo_on_click(self, *args):
+        """
+        Callback to button for attempting to find center automatically using
+        `tomopy.recon.rotation.find_center_vo`. Note: this method has not worked
+        well for me.
+        """
+        prj_imgs, ds_value = self.get_ds_projections()
+        angles_rad = self.projections.angles_rad
+        try:
+            self.current_center = find_center_vo(prj_imgs, ncore=1)
+            self.center_textbox.value = self.current_center
+        except Excepton:
+            self.find_center_vo_button.warning()
+
+    def find_center_manual_on_click(self, *args):
+        """
+        Reconstructs at various centers when you click the button, and plots
+        the results with a slider so one can view. TODO: see X example.
+        Uses search_range, search_step, center_guess.
+        Creates a :doc:`hyperslicer <mpl-interactions:examples/hyperslicer>` +
+        :doc:`histogram <mpl-interactions:examples/hist>` plot
+        """
+        prj_imgs, ds_value = self.get_ds_projections()
+        angles_rad = self.projections.angles_rad
+        ds_factor = np.power(2, int(ds_value + 1))
+        _center_guess = copy.deepcopy(self.center_guess) / ds_factor
+        _search_range = copy.deepcopy(self.search_range) / ds_factor
+        _search_step = copy.deepcopy(self.search_step) / ds_factor
+        _index_to_try = int(copy.deepcopy(self.index_to_try) / ds_factor)
+        cen_range = [
+            _center_guess - _search_range,
+            _center_guess + _search_range,
+            _search_step,
+        ]
+        # reconstruct, but also pull the centers used out to map to center
+        # textbox
+        self.rec, cen_range = write_center(
+            prj_imgs,
+            angles_rad,
+            cen_range=cen_range,
+            ind=_index_to_try,
+            mask=True,
+            algorithm=self.algorithm,
+            filter_name=self.filter,
+            num_iter=self.num_iter,
+        )
+        self.cen_range = [ds_factor * cen for cen in cen_range]
+        if self.rec is None:
+            self.find_center_manual_button.warning()
+            return
+        self.rec_viewer.plot(self.rec)
+
+    def get_ds_projections(self):
+        ds_value = self.viewer.ds_viewer_dropdown.value
+        if self.use_ds:
+            if ds_value == -1:
+                prj_imgs = self.projections.data
+            else:
+                self.projections._load_hdf_ds_data_into_memory(pyramid_level=ds_value)
+                prj_imgs = self.projections.data_ds
+        elif ds_value == -1:
+            prj_imgs = self.projections.data
+        else:
+            self.projections._load_hdf_normalized_data_into_memory()
+            prj_imgs = self.projections.data
+        return prj_imgs, ds_value
 
     def refresh_plots(self):
-        self.projection_viewer.plot()
+        self.viewer.plot(self.projections)
+        self._load_rough_center_onclick(None)
+        self.find_center_button.enable()
+        self.find_center_manual_button.enable()
+        self.find_center_vo_button.enable()
 
     def make_tab(self):
         """
@@ -359,7 +352,7 @@ class Center:
         self.automatic_center_vbox = VBox(
             [
                 HBox(
-                    [self.find_center_button, self.find_center_vo_button],
+                    [self.find_center_button.button, self.find_center_vo_button.button],
                     layout=Layout(justify_content="center"),
                 ),
                 HBox(
@@ -381,11 +374,11 @@ class Center:
         self.manual_center_vbox = VBox(
             [
                 HBox(
-                    [self.projection_viewer.app, self.rec_viewer.app],
+                    [self.viewer.app, self.rec_viewer.app],
                     layout=Layout(justify_content="center"),
                 ),
                 HBox(
-                    [self.find_center_manual_button],
+                    [self.find_center_manual_button.button],
                     layout=Layout(justify_content="center"),
                 ),
                 HBox(
@@ -397,6 +390,7 @@ class Center:
                         self.search_step_textbox,
                         self.algorithms_dropdown,
                         self.filters_dropdown,
+                        self.use_ds_checkbox,
                     ],
                     layout=Layout(
                         display="flex",
